@@ -353,8 +353,6 @@ soup_message_body_new (void)
  * In either case, @body's %data field will not be filled in after the
  * body is fully sent/received, since the body data will no longer be
  * available
- *
- * Since: 2.4.1
  **/
 void
 soup_message_body_set_accumulate (SoupMessageBody *body,
@@ -373,8 +371,6 @@ soup_message_body_set_accumulate (SoupMessageBody *body,
  * soup_message_body_set_accumulate() for details.
  *
  * Return value: the accumulate flag for @body.
- *
- * Since: 2.4.1
  **/
 gboolean
 soup_message_body_get_accumulate (SoupMessageBody *body)
@@ -582,8 +578,6 @@ soup_message_body_get_chunk (SoupMessageBody *body, goffset offset)
  *
  * This is a low-level method which you should not normally need to
  * use.
- *
- * Since: 2.4.1
  **/
 void
 soup_message_body_got_chunk (SoupMessageBody *body, SoupBuffer *chunk)
@@ -599,18 +593,15 @@ soup_message_body_got_chunk (SoupMessageBody *body, SoupBuffer *chunk)
 /**
  * soup_message_body_wrote_chunk:
  * @body: a #SoupMessageBody
- * @chunk: a #SoupBuffer returned from soup_message_body_get_chunk()
+ * @chunk: a #SoupBuffer received from the network
  *
  * Handles the #SoupMessageBody part of writing a chunk of data to the
  * network. Normally this is a no-op, but if you have set @body's
- * accumulate flag to %FALSE, then this will cause @chunk to be
- * discarded to free up memory.
+ * accumulate flag to %FALSE, then this will cause @chunk (and any
+ * chunks preceding it in @body) to be discarded to free up memory.
  *
- * This is a low-level method which you should not need to use, and
- * there are further restrictions on its proper use which are not
- * documented here.
- *
- * Since: 2.4.1
+ * This is a low-level method which you should not normally need to
+ * use.
  **/
 void
 soup_message_body_wrote_chunk (SoupMessageBody *body, SoupBuffer *chunk)
@@ -621,16 +612,15 @@ soup_message_body_wrote_chunk (SoupMessageBody *body, SoupBuffer *chunk)
 	if (priv->accumulate)
 		return;
 
-	chunk2 = priv->chunks->data;
-	g_return_if_fail (chunk->length == chunk2->length);
-	g_return_if_fail (chunk == chunk2 || ((SoupBufferPrivate *)chunk2)->use == SOUP_MEMORY_TEMPORARY);
+	do {
+		chunk2 = priv->chunks->data;
+		priv->chunks = g_slist_remove (priv->chunks, chunk2);
+		priv->base_offset += chunk2->length;
+		soup_buffer_free (chunk2);
+	} while (priv->chunks && chunk2 != chunk);
 
-	priv->chunks = g_slist_remove (priv->chunks, chunk2);
 	if (!priv->chunks)
 		priv->last = NULL;
-
-	priv->base_offset += chunk2->length;
-	soup_buffer_free (chunk2);
 }
 
 static SoupMessageBody *
